#!/usr/bin/perl -w
#
# $Id: gen-unicode-strings.pl,v 1.3 2003/12/29 22:38:46 nlevitt Exp $ 
#
# see usage notes below
#

use strict;
use vars ('$UNZIP', '$ICONV', '$UNICODE_VERSION');

# if these things aren't in your path you can put full paths to them here
$UNZIP = 'unzip';
$ICONV = 'iconv';

sub process_unicode_data_txt ($);
sub process_unihan_zip ($);
sub process_nameslist_txt ($);
sub process_blocks_txt ($);
sub process_scripts_txt ($);
sub preamble ($$);

$| = 1;  # flush stdout buffer

if (@ARGV != 2) 
{
    $0 =~ s@.*/@@;
    die <<EOF

Usage: $0 UNICODE-VERSION DIRECTORY

DIRECTORY should contain the following Unicode data files:
NamesList.txt Blocks.txt Scripts.txt

which can be found at http://www.unicode.org/Public/UNIDATA/

EOF
}

my ($unicodedata_txt, $unihan_zip, $nameslist_txt, $blocks_txt, $scripts_txt);

$UNICODE_VERSION = $ARGV[0];
my $d = $ARGV[1];
opendir (my $dir, $d) or die "Cannot open Unicode data dir $d: $!\n";
for my $f (readdir ($dir))
{
    $nameslist_txt = "$d/$f" if ($f =~ /NamesList.*\.txt/);
    $blocks_txt = "$d/$f" if ($f =~ /Blocks.*\.txt/);
    $scripts_txt = "$d/$f" if ($f =~ /Scripts.*\.txt/);
}

defined $nameslist_txt or die "Did not find $d/NamesList.txt";
defined $blocks_txt or die "Did not find $d/Blocks.txt";
defined $scripts_txt or die "Did not find $d/Scripts.txt";

process_nameslist_txt ($nameslist_txt);
process_blocks_txt ($blocks_txt);
process_scripts_txt ($scripts_txt);

exit;


#------------------------#

sub preamble ($$)
{
    my ($fh, $filename) = @_;

    print $fh "/* $filename */\n";
    print $fh "/* THIS IS A GENERATED FILE. CHANGES WILL BE OVERWRITTEN. */\n";
    print $fh "/* This file is part of the unicode-translation project. */\n";
    print $fh "/* Generated by $0 */\n";
    print $fh "\n";
    print $fh "/*\n";
    print $fh " * This file isn't meant to be used as real C code. It is for gettext to\n";
    print $fh " * parse and create the pot file. I'm doing it this way so that gettext can\n";
    print $fh " * work its magic with fuzzy strings and whatnot. This could change in the\n";
    print $fh " * future.\n";
    print $fh " */\n";
    print $fh "\n";
    print $fh "/* Prevent the above comments from showing up in the pot file */\n";
    print $fh ";\n";
    print $fh "\n";
    print $fh "/* Unicode version $UNICODE_VERSION. */\n";
    print $fh "/* */\n";
    print $fh "/* Translators: you might want to consult gucharmap if you are unsure of */\n";
    print $fh "/* the context for a particular string. */\n";
    print $fh "/* */\n";
    print $fh "\n";
}

#------------------------#

sub process_nameslist_txt ($)
{
    my ($nameslist_txt) = @_;

    open (my $nameslist, "$ICONV -f 'ISO8859-1' -t 'UTF-8' $nameslist_txt |") or die;

    open (my $names, "> unicode-names.h") or die;
    open (my $aliases, "> unicode-aliases.h") or die;
    open (my $notes, "> unicode-notes.h") or die;

    print "processing $nameslist_txt...";

    preamble ($names, "unicode-names.h");
    preamble ($aliases, "unicode-aliases.h");
    preamble ($notes, "unicode-notes.h");

    my ($wc, $name);

    while (my $line = <$nameslist>)
    {
        chomp ($line);

        if ($line =~ /^@/)
        {
            next;
        }
        elsif ($line =~ /^([0-9A-F]+)\s+(.+)$/)
        {
            $wc = $1;
            $name = $2;

            print $names qq{/* CHARACTER NAME for U+$wc */\n};
            print $names qq{N_("$name");\n\n};
        }
        elsif ($line =~ /^\s+=\s+(.+)$/)
        {
	    my $value = $1;
	    $value =~ s/\"/\\"/g;

            print $aliases qq{/* ALIAS NAME for U+$wc $name */\n};
            print $aliases qq{N_("$value");\n\n};
        }
        elsif ($line =~ /^\s+\*\s+(.+)$/)
        {
	    my $value = $1;
	    $value =~ s/\"/\\"/g;

            print $notes qq{/* EXPLANATORY NOTE for U+$wc $name */\n};
            print $notes qq{N_("$value");\n\n};
        }
    }

    close ($nameslist);

    close ($names);
    close ($aliases);
    close ($notes);

    print " done.\n";
}

#------------------------#

sub process_blocks_txt ($)
{
    my ($blocks_txt) = @_;

    open (my $blocks, $blocks_txt) or die;
    open (my $out, "> unicode-blocks.h") or die;

    print "processing $blocks_txt...";

    preamble ($out, "unicode-blocks.h");

    while (my $line = <$blocks>)
    {
        $line =~ /^([0-9A-F]+)\.\.([0-9A-F]+); (.+)$/ or next;
        print $out qq|/* UNICODE BLOCK: U+$1..U+$2 */\n|;
        print $out qq|N_("$3");\n\n|;
    }

    close ($blocks);
    close ($out);

    print " done.\n";
}


#------------------------#

sub process_scripts_txt ($)
{
    my ($scripts_txt) = @_;
    my %scripts;

    open (my $in, $scripts_txt) or die;

    print "processing $scripts_txt...";

    while (my $line = <$in>)
    {
        chomp ($line);

        my $raw_script;

        if ($line =~ /^([0-9A-F]+)\.\.([0-9A-F]+)\s+;\s+(\S+)/)
        {
            $raw_script = $3;
        }
        elsif ($line =~ /^([0-9A-F]+)\s+;\s+(\S+)/)
        {
            $raw_script = $2;
        }
        else 
        {
            next;
        }

        my $script = $raw_script;
        $script =~ tr/_/ /;
        $script =~ s/(\w+)/\u\L$1/g;

        $scripts{$raw_script} = $script;
    }

    close ($in);

    open (my $out, "> unicode-scripts.h") or die;
    preamble ($out, "unicode-scripts.h");

    for my $raw_script (sort (keys %scripts))
    {
        print $out qq|/* UNICODE SCRIPT: "$raw_script" */\n|;
        print $out qq|N_("$scripts{$raw_script}");\n\n|;
    }

    close ($out);

    print " done.\n";
}
